
From: Andi Kleen <ak@suse.de>

When the user gave an address hint in mmap use it as starting point for the
search for !MAP_FIXED.

Currently it is only checked directly and when already used the free area
cache is used as starting point.  With this change you can use mmap(4096,
....) to e.g.  get the lowest free address in your address space, which is
sometimes useful.  For example on x86-64 glibc wants to preferably allocate
thread local data in the first 4GB but use higher addresses when this is
not possible.

This can be a bit more costly in CPU time because it may have to skip over
more VMAs, but gives better semantics for most cases.  Most programs pass
NULL as hint anyways so it won't make any difference for them.  

I did it for the generic mmap and for x86-64 for now.  Also minor white
space fixes for x86-64.



---

 25-akpm/arch/x86_64/kernel/sys_x86_64.c |    6 +++---
 25-akpm/mm/mmap.c                       |    5 +++--
 2 files changed, 6 insertions(+), 5 deletions(-)

diff -puN arch/x86_64/kernel/sys_x86_64.c~mmap-use-address-hint arch/x86_64/kernel/sys_x86_64.c
--- 25/arch/x86_64/kernel/sys_x86_64.c~mmap-use-address-hint	Wed Jan 28 13:38:24 2004
+++ 25-akpm/arch/x86_64/kernel/sys_x86_64.c	Wed Jan 28 13:38:24 2004
@@ -105,13 +105,13 @@ arch_get_unmapped_area(struct file *filp
 		return -ENOMEM;
 
 	if (addr) {
-	addr = PAGE_ALIGN(addr);
+		addr = PAGE_ALIGN(addr);
 		vma = find_vma(mm, addr);
 		if (end - len >= addr &&
 		    (!vma || addr + len <= vma->vm_start))
 			return addr;
-	}
-	addr = mm->free_area_cache;
+	} else
+		addr = mm->free_area_cache;
 	if (addr < begin) 
 		addr = begin; 
 	start_addr = addr;
diff -puN mm/mmap.c~mmap-use-address-hint mm/mmap.c
--- 25/mm/mmap.c~mmap-use-address-hint	Wed Jan 28 13:38:24 2004
+++ 25-akpm/mm/mmap.c	Wed Jan 28 13:38:24 2004
@@ -743,8 +743,9 @@ arch_get_unmapped_area(struct file *filp
 		if (TASK_SIZE - len >= addr &&
 		    (!vma || addr + len <= vma->vm_start))
 			return addr;
-	}
-	start_addr = addr = mm->free_area_cache;
+	} else
+		addr = mm->free_area_cache;
+	start_addr = addr;
 
 full_search:
 	for (vma = find_vma(mm, addr); ; vma = vma->vm_next) {

_
